@@ -1,4 +1,4 @@
-// Copyright 2019-2020 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -25,6 +25,7 @@ import (
 	"github.com/gcla/termshark/v2/pcap"
 	"github.com/gcla/termshark/v2/streams"
 	"github.com/gcla/termshark/v2/system"
+	"github.com/gcla/termshark/v2/theme"
 	"github.com/gcla/termshark/v2/tty"
 	"github.com/gcla/termshark/v2/ui"
 	"github.com/gcla/termshark/v2/widgets/filter"
@@ -34,7 +35,6 @@ import (
 	"github.com/shibukawa/configdir"
 	log "github.com/sirupsen/logrus"
 	"github.com/spf13/viper"
-	"gopkg.in/fsnotify.v1"
 
 	"net/http"
 	_ "net/http"
@@ -270,6 +270,7 @@ func cmain() int {
 	// terminal emumlator supports 256 colors.
 	termVar := termshark.ConfString("main.term", "")
 	if termVar != "" {
+		log.Infof("Configuration file overrides TERM setting, using TERM=%s", termVar)
 		os.Setenv("TERM", termVar)
 	}
 
@@ -290,7 +291,7 @@ func cmain() int {
 	// If no interface specified, and no pcap specified via -r, then we assume the first
 	// argument is a pcap file e.g. termshark foo.pcap
 	if pcapf == "" && len(opts.Ifaces) == 0 {
-		pcapf = string(opts.Args.FilterOrFile)
+		pcapf = string(opts.Args.FilterOrPcap)
 		// `termshark` => `termshark -i 1` (livecapture on default interface if no args)
 		if pcapf == "" {
 			if termshark.IsTerminal(os.Stdin.Fd()) {
@@ -325,7 +326,7 @@ func cmain() int {
 		}
 	} else {
 		// Add it to filter args. Figure out later if they're capture or display.
-		filterArgs = append(filterArgs, opts.Args.FilterOrFile)
+		filterArgs = append(filterArgs, opts.Args.FilterOrPcap)
 	}
 
 	if pcapf != "" && len(opts.Ifaces) > 0 {
@@ -345,6 +346,62 @@ func cmain() int {
 		}
 	}
 
+	// Here we check for
+	// (a) sources named '-' - these need rewritten to /dev/fd/N and stdin needs to be moved
+	// (b) fifo sources - these are switched from -r to -i because that's what tshark needs
+	haveStdin := false
+	for pi, psrc := range psrcs {
+		switch {
+		case psrc.Name() == "-":
+			if haveStdin {
+				fmt.Fprintf(os.Stderr, "Requested live capture %v (\"stdin\") cannot be supplied more than once.\n", psrc.Name())
+				return 1
+			}
+
+			if termshark.IsTerminal(os.Stdin.Fd()) {
+				fmt.Fprintf(os.Stderr, "Requested live capture is %v (\"stdin\") but stdin is a tty.\n", psrc.Name())
+				fmt.Fprintf(os.Stderr, "Perhaps you intended to pipe packet input to termshark?\n")
+				return 1
+			}
+			if runtime.GOOS != "windows" {
+				psrcs[pi] = pcap.PipeSource{Descriptor: "/dev/fd/0", Fd: int(os.Stdin.Fd())}
+				haveStdin = true
+			} else {
+				fmt.Fprintf(os.Stderr, "Sorry, termshark does not yet support piped input on Windows.\n")
+				return 1
+			}
+		default:
+			stat, err := os.Stat(psrc.Name())
+			if err != nil {
+				if psrc.IsFile() || psrc.IsFifo() {
+					// Means this was supplied with -r - since any file sources means there's (a) 1 and (b)
+					// no other sources. So it must stat. Note if we started with -i fifo, this check
+					// isn't done... but it still ought to exist.
+					fmt.Fprintf(os.Stderr, "Error reading file %s: %v.\n", psrc.Name(), err)
+					return 1
+				}
+				continue
+			}
+			if stat.Mode()&os.ModeNamedPipe != 0 {
+				// If termshark was invoked with -r myfifo, switch to -i myfifo, which tshark uses. This
+				// also puts termshark in "interface" mode where it assumes the source is unbounded
+				// (e.g. a different spinner)
+				psrcs[pi] = pcap.FifoSource{Filename: psrc.Name()}
+			} else {
+				if pcapffile, err := os.Open(psrc.Name()); err != nil {
+					// Do this up front before the UI starts to catch simple errors quickly - like
+					// the file not being readable. It's possible that tshark would be able to read
+					// it and the termshark user not, but unlikely.
+					fmt.Fprintf(os.Stderr, "Error reading file %s: %v.\n", psrc.Name(), err)
+					return 1
+				} else {
+					pcapffile.Close()
+				}
+			}
+		}
+	}
+
+	// Means files
 	fileSrcs := pcap.FileSystemSources(psrcs)
 	if len(fileSrcs) == 1 {
 		if len(psrcs) > 1 {
@@ -394,70 +451,6 @@ func cmain() int {
 		}
 	}
 
-	// - means read from stdin. But termshark uses stdin for interacting with the UI. So if the
-	// iface is -, then dup stdin to a free descriptor, adjust iface to read from that descriptor,
-	// then open /dev/tty on stdin.
-	newinputfd := -1
-
-	// Here we check for
-	// (a) sources named '-' - these need rewritten to /dev/fd/N and stdin needs to be moved
-	// (b) fifo sources
-	renamedStdin := false
-	for pi, psrc := range psrcs {
-		switch {
-		case psrc.Name() == "-":
-			if renamedStdin {
-				fmt.Fprintf(os.Stderr, "Requested live capture %v (\"stdin\") cannot be supplied more than once.\n", psrc.Name())
-				return 1
-			}
-			if termshark.IsTerminal(os.Stdin.Fd()) {
-				fmt.Fprintf(os.Stderr, "Requested live capture is %v (\"stdin\") but stdin is a tty.\n", psrc.Name())
-				fmt.Fprintf(os.Stderr, "Perhaps you intended to pipe packet input to termshark?\n")
-				return 1
-			}
-			if runtime.GOOS != "windows" {
-				newinputfd, err = system.MoveStdin()
-				if err != nil {
-					fmt.Fprintf(os.Stderr, "%v\n", err)
-					return 1
-				}
-				psrcs[pi] = pcap.PipeSource{Descriptor: fmt.Sprintf("/dev/fd/%d", newinputfd), Fd: newinputfd}
-				renamedStdin = true
-			} else {
-				fmt.Fprintf(os.Stderr, "Sorry, termshark does not yet support piped input on Windows.\n")
-				return 1
-			}
-		default:
-			stat, err := os.Stat(psrc.Name())
-			if err != nil {
-				if len(fileSrcs) > 0 {
-					// Means this was supplied with -r - since any file sources means there's (a) 1 and (b)
-					// no other sources. So it must stat. Note if we started with -i fifo, this check
-					// isn't done... but it still ought to exist.
-					fmt.Fprintf(os.Stderr, "Error reading file %s: %v.\n", psrc.Name(), err)
-					return 1
-				}
-				continue
-			}
-			if stat.Mode()&os.ModeNamedPipe != 0 {
-				// If termshark was invoked with -r myfifo, switch to -i myfifo, which tshark uses. This
-				// also puts termshark in "interface" mode where it assumes the source is unbounded
-				// (e.g. a different spinner)
-				psrcs[pi] = pcap.FifoSource{Filename: psrc.Name()}
-			} else {
-				if pcapffile, err := os.Open(psrc.Name()); err != nil {
-					// Do this up front before the UI starts to catch simple errors quickly - like
-					// the file not being readable. It's possible that tshark would be able to read
-					// it and the termshark user not, but unlikely.
-					fmt.Fprintf(os.Stderr, "Error reading file %s: %v.\n", psrc.Name(), err)
-					return 1
-				} else {
-					pcapffile.Close()
-				}
-			}
-		}
-	}
-
 	// Here we now have an accurate view of all psrcs - either file, fifo, pipe or interface
 
 	// Helpful to use logging when enumerating interfaces below, so do it first
@@ -475,7 +468,12 @@ func cmain() int {
 		log.SetOutput(logfd)
 	}
 
-	if cli.FlagIsTrue(opts.Debug) {
+	debug := false
+	if (opts.Debug.Set && opts.Debug.Val == true) || (!opts.Debug.Set && termshark.ConfBool("main.debug", false)) {
+		debug = true
+	}
+
+	if debug {
 		for _, addr := range termshark.LocalIPs() {
 			log.Infof("Starting debug web server at http://%s:6060/debug/pprof/", addr)
 		}
@@ -642,7 +640,7 @@ func cmain() int {
 		// fifo. In all cases, we save the packets to a file so that if a
 		// filter is applied, we can restart - and so that we preserve the
 		// capture at the end of running termshark.
-		if len(fileSrcs) == 0 && startedSuccessfully {
+		if len(pcap.FileSystemSources(psrcs)) == 0 && startedSuccessfully {
 			fmt.Printf("Packets read from %s have been saved in %s\n", pcap.SourcesString(psrcs), ifacePcapFilename)
 		}
 	}()
@@ -693,61 +691,59 @@ func cmain() int {
 		log.Infof("Config specifies pcap-bundle-size as %d - setting to max (%d)", bundleSize, maxBundleSize)
 		bundleSize = maxBundleSize
 	}
-	ui.PcapScheduler = pcap.NewScheduler(
-		pcap.MakeCommands(opts.DecodeAs, tsharkArgs, pdmlArgs, psmlArgs, ui.PacketColors),
-		pcap.Options{
-			CacheSize:      cacheSize,
-			PacketsPerLoad: bundleSize,
-		},
-	)
-	ui.Loader = ui.PcapScheduler.Loader
 
-	// Buffered because I might send something in this goroutine
-	startUIChan := make(chan struct{}, 1)
-	// Used to cancel the display of a message telling the user why there is no UI yet.
-	detectMsgChan := make(chan struct{}, 1)
-
-	var iwatcher *fsnotify.Watcher
 	var ifaceTmpFile string
 
-	if len(fileSrcs) == 0 {
+	if len(pcap.FileSystemSources(psrcs)) == 0 {
 		srcNames := make([]string, 0, len(psrcs))
 		for _, psrc := range psrcs {
 			srcNames = append(srcNames, psrc.Name())
 		}
 		ifaceTmpFile = pcap.TempPcapFile(srcNames...)
 
-		iwatcher, err = fsnotify.NewWatcher()
-		if err != nil {
-			fmt.Fprintf(os.Stderr, "Could not start filesystem watcher: %v\n", err)
-			return 1
-		}
-		defer func() {
-			if iwatcher != nil {
-				iwatcher.Close()
-			}
-		}()
-
-		// Don't start the UI until this file is created. When listening on a pipe,
-		// termshark will start a process similar to:
-		//
-		// dumpcap -i /dev/fd/3 -w ~/.cache/pcaps/tmp123.pcap
-		//
-		// dumpcap will not actually create that file until it has data to write to it.
-		// So we watch for the creation of that file, and until then, don't launch the UI.
-		// Then if the feeding process needs input first e.g. sudo tcpdump needs password,
-		// there won't be a conflict for reading /dev/tty.
-		//
-		if err := iwatcher.Add(termshark.PcapDir()); err != nil { //&& !os.IsNotExist(err) {
-			fmt.Fprintf(os.Stderr, "Could not set up watcher for %s: %v\n", termshark.PcapDir(), err)
-			return 1
-		}
-
 		fmt.Printf("(The termshark UI will start when packets are detected...)\n")
-
 	} else {
 		// Start UI right away, reading from a file
-		startUIChan <- struct{}{}
+		close(ui.StartUIChan)
+	}
+
+	// Need to figure out possible changes to COLORTERM before creating the
+	// tcell screen. Note that even though apprunner.Start() below will create
+	// a new screen, it will use a terminfo that it constructed the first time
+	// we call NewApp(), because tcell stores these in a global map. So if the
+	// first terminfo is created in an environment with COLORTERM=truecolor,
+	// the terminfo Go struct is extended with codes that emit truecolor-compatible
+	// ansi codes for colors. Then if I later create a new screen without COLORTERM,
+	// tcell will still use the extended terminfo struct and emit truecolor-codes
+	// anyway.
+	//
+	// If you are using base16-shell, the lowest colors 0-21 in the 256 color space
+	// will be remapped to whatever colors the terminal base16 theme sets up. If you
+	// are using a termshark theme that expresses colors in RGB style (#7799AA), and
+	// termshark is running in a 256-color terminal, then termshark will find the closest
+	// match for the RGB color in the 256 color-space. But termshark assumes that colors
+	// 0-21 are set up normally, and not remapped. If the closest match is one of those
+	// colors, then the theme won't look as expected. A workaround is to tell
+	// gowid not to use colors 0-21 when finding the closest match.
+	if termshark.ConfKeyExists("main.ignore-base16-colors") {
+		gowid.IgnoreBase16 = termshark.ConfBool("main.ignore-base16-colors", false)
+	} else {
+		// Try to auto-detect whether or not base16-shell is installed and in-use
+		gowid.IgnoreBase16 = (os.Getenv("BASE16_SHELL") != "")
+	}
+	if gowid.IgnoreBase16 {
+		log.Infof("Will not consider colors 0-21 from the terminal 256-color-space when interpolating theme colors")
+		// If main.respect-colorterm=true then termshark will leave COLORTERM set and use
+		// 24-bit color if possible. The problem with this, in the presence of base16, is that
+		// some terminal-emulators - e.g. gnome-terminal - still seems to map RGB ANSI codes
+		// colors that are set at values 0-21 in the 256-color space. I'm not sure if this is
+		// just an implementation snafu, or if something else is going on... In any case,
+		// termshark will fall back to 256-colors if base16 is detected because I can
+		// programmatically avoid choosing colors 0-21 for anything termshark needs.
+		if os.Getenv("COLORTERM") != "" && !termshark.ConfBool("main.respect-colorterm", false) {
+			log.Infof("Pessimistically disabling 24-bit color to avoid conflicts with base16")
+			os.Unsetenv("COLORTERM")
+		}
 	}
 
 	// Do this before ui.Build. If ui.Build fails (e.g. bad TERM), then the filter will be left
@@ -772,6 +768,16 @@ func cmain() int {
 
 	appRunner := app.Runner()
 
+	pcap.PcapCmds = pcap.MakeCommands(opts.DecodeAs, tsharkArgs, pdmlArgs, psmlArgs, ui.PacketColors)
+	pcap.PcapOpts = pcap.Options{
+		CacheSize:      cacheSize,
+		PacketsPerLoad: bundleSize,
+	}
+
+	// This is a global. The type supports swapping out the real loader by embedding it via
+	// pointer, but I assume this only happens in the main goroutine.
+	ui.Loader = &pcap.PacketLoader{ParentLoader: pcap.NewPcapLoader(pcap.PcapCmds, &pcap.Runner{app}, pcap.PcapOpts)}
+
 	// Populate the filter widget initially - runs asynchronously
 	go ui.FilterWidget.UpdateCompletions(app)
 
@@ -783,7 +789,7 @@ func cmain() int {
 			Fn: func(app gowid.IApp) {
 				if !ui.Running {
 					fmt.Fprintf(os.Stderr, "Invalid filter: %s\n", displayFilter)
-					ui.QuitRequestedChan <- struct{}{}
+					ui.RequestQuit()
 				} else {
 					app.Run(gowid.RunFunction(func(app gowid.IApp) {
 						ui.OpenError(fmt.Sprintf("Invalid filter: %s", displayFilter), app)
@@ -793,6 +799,8 @@ func cmain() int {
 		},
 	}
 
+	// Refresh
+	fileSrcs = pcap.FileSystemSources(psrcs)
 	if len(fileSrcs) > 0 {
 		psrc := fileSrcs[0]
 		absfile, err := filepath.Abs(psrc.Name())
@@ -805,12 +813,10 @@ func cmain() int {
 			app.Run(gowid.RunFunction(func(app gowid.IApp) {
 				ui.FilterWidget.SetValue(displayFilter, app)
 			}))
-			ui.RequestLoadPcapWithCheck(absfile, displayFilter, app)
+			ui.RequestLoadPcapWithCheck(absfile, displayFilter, ui.NoGlobalJump, app)
 		}
 		validator.Valid = &filter.ValidateCB{Fn: doit, App: app}
 		validator.Validate(displayFilter)
-		// no auto-scroll when reading a file
-		ui.AutoScroll = false
 	} else {
 
 		// Verifies whether or not we will be able to read from the interface (hopefully)
@@ -836,30 +842,23 @@ func cmain() int {
 				ui.FilterWidget.SetValue(displayFilter, app)
 			}))
 			ifacePcapFilename = ifaceTmpFile
-			ui.PcapScheduler.RequestLoadInterfaces(psrcs, captureFilter, displayFilter, ifaceTmpFile,
-				pcap.HandlerList{
-					ui.MakeSaveRecents("", displayFilter, app),
-					ui.MakePacketViewUpdater(app),
-					ui.MakeUpdateCurrentCaptureInTitle(app),
-					ui.ManageStreamCache{},
-				},
-			)
+			ui.RequestLoadInterfaces(psrcs, captureFilter, displayFilter, ifaceTmpFile, app)
 		}
 		validator.Valid = &filter.ValidateCB{Fn: ifValid, App: app}
 		validator.Validate(displayFilter)
 	}
 
-	quitRequested := false
 	quitIssuedToApp := false
-	prevstate := ui.Loader.State()
-	var prev float64
+
+	wasLoadingPdmlLastTime := ui.Loader.PdmlLoader.IsLoading()
+	wasLoadingAnythingLastTime := ui.Loader.LoadingAnything()
+
+	// Keep track of this across runs of the main loop so we don't go backwards (because
+	// that looks wrong to the user)
+	var prevProgPercentage float64
 
 	progTicker := time.NewTicker(time.Duration(200) * time.Millisecond)
 
-	loaderPsmlFinChan := ui.Loader.PsmlFinishedChan
-	loaderIfaceFinChan := ui.Loader.IfaceFinishedChan
-	loaderPdmlFinChan := ui.Loader.Stage2FinishedChan
-
 	ctrlzLineDisc := tty.TerminalSignals{}
 
 	// This is used to stop iface load and any stream reassembly. Make sure to
@@ -870,27 +869,21 @@ func cmain() int {
 		if ui.StreamLoader != nil {
 			ui.StreamLoader.SuppressErrors = true
 		}
-		ui.Loader.SuppressErrors = true
-		ui.Loader.Close()
+		ui.Loader.CloseMain()
 	}
 
 	inactiveDuration := 30 * time.Second
 	inactivityTimer := time.NewTimer(inactiveDuration)
 
+	var progCancelTimer *time.Timer
+
 Loop:
 	for {
 		var finChan <-chan time.Time
-		var opsChan <-chan pcap.RunFn
 		var tickChan <-chan time.Time
 		var inactivityChan <-chan time.Time
-		var emptyStructViewChan <-chan time.Time
-		var emptyHexViewChan <-chan time.Time
-		var psmlFinChan <-chan struct{}
-		var ifaceFinChan <-chan struct{}
-		var pdmlFinChan <-chan struct{}
-		var tmpPcapWatcherChan <-chan fsnotify.Event
-		var tmpPcapWatcherErrorsChan <-chan error
 		var tcellEvents <-chan tcell.Event
+		var opsChan <-chan gowid.RunFunction
 		var afterRenderEvents <-chan gowid.IAfterRenderEvent
 		// For setting struct views empty. This isn't done as soon as a load is initiated because
 		// in the case we are loading from an interface and following new packets, we get an ugly
@@ -900,17 +893,14 @@ Loop:
 		// beginning to get new packets). Waiting 500ms to display loading gives enough time, in
 		// practice,
 
-		if ui.EmptyStructViewTimer != nil {
-			emptyStructViewChan = ui.EmptyStructViewTimer.C
-		}
-		// For setting hex views empty
-		if ui.EmptyHexViewTimer != nil {
-			emptyHexViewChan = ui.EmptyHexViewTimer.C
+		// On change of state - check for new pdml requests
+		if ui.Loader.PdmlLoader.IsLoading() != wasLoadingPdmlLastTime {
+			ui.CacheRequestsChan <- struct{}{}
 		}
 
 		// This should really be moved to a handler...
-		if ui.Loader.State() == 0 {
-			if prevstate != 0 {
+		if !ui.Loader.LoadingAnything() {
+			if wasLoadingAnythingLastTime {
 				// If the state has just switched to 0, it means no interface-reading process is
 				// running. That means we will no longer be reading from an interface or a fifo, so
 				// we point the loader at the file we wrote to the cache, and redirect all
@@ -920,14 +910,18 @@ Loop:
 					ui.ClearProgressWidget(app)
 					ui.SetProgressDeterminate(app) // always switch back - for pdml (partial) loads of later data.
 				}))
+
 				// When the progress bar is enabled, track the previous percentage reached. This is
 				// so that I don't go "backwards" if I generate a progress value less than the last
 				// one, using the current algorithm (because it would be confusing to see it go
 				// backwards)
-				prev = 0.0
+				prevProgPercentage = 0.0
 			}
 
-			if quitRequested {
+			// EnableOpsVar will be enabled when all the handlers have run, which happen in the main goroutine.
+			// I need them to run because the loader channel is closed in one, and the ticker goroutines
+			// don't terminate until these goroutines stop
+			if ui.QuitRequested {
 				if ui.Running {
 					if !quitIssuedToApp {
 						app.Quit()
@@ -940,37 +934,20 @@ Loop:
 			}
 		}
 
-		if ui.Loader.State()&(pcap.LoadingPdml|pcap.LoadingPsml) != 0 {
+		// Only display the progress bar if PSML is loading or if PDML is loading that is needed
+		// by the UI. If the PDML is an optimistic load out of the display, then no need for
+		// progress.
+		if ui.Loader.PsmlLoader.IsLoading() || (ui.Loader.PdmlLoader.IsLoading() && ui.Loader.PdmlLoader.LoadIsVisible()) {
 			tickChan = progTicker.C // progress is only enabled when a pcap may be loading
+		} else {
+			// Reset for the next load
+			prevProgPercentage = 0.0
 		}
 
-		if ui.Loader.State()&pcap.LoadingPdml != 0 {
-			pdmlFinChan = loaderPdmlFinChan
-		}
-
-		if ui.Loader.State()&pcap.LoadingPsml != 0 {
-			psmlFinChan = loaderPsmlFinChan
-		}
-
-		if ui.Loader.State()&pcap.LoadingIface != 0 {
-			ifaceFinChan = loaderIfaceFinChan
+		if ui.Loader.InterfaceLoader.IsLoading() {
 			inactivityChan = inactivityTimer.C
 		}
 
-		// (User) operations are enabled by default (the test predicate is nil), or if the predicate returns true
-		// meaning the operation has reached its desired state. Only one operation can be in progress at a time.
-		if ui.PcapScheduler.IsEnabled() {
-			opsChan = ui.PcapScheduler.OperationsChan
-		}
-
-		// This tracks a temporary pcap file which is populated by dumpcap when termshark is
-		// reading from a fifo. If iwatcher is nil, it means we've got data and don't need to
-		// monitor any more.
-		if iwatcher != nil {
-			tmpPcapWatcherChan = iwatcher.Events
-			tmpPcapWatcherErrorsChan = iwatcher.Errors
-		}
-
 		// Only process tcell and gowid events if the UI is running.
 		if ui.Running {
 			tcellEvents = app.TCellEvents
@@ -980,9 +957,19 @@ Loop:
 			finChan = ui.Fin.C()
 		}
 
+		// For operations like ClearPcap - need previous loads to be fully finished first. The operations
+		// channel is enabled until an operation starts, then disabled until the operation re-enables it
+		// via a handler.
+		//
+		// Make sure state doesn't change until all handlers have been run
+		if !ui.Loader.PdmlLoader.IsLoading() && !ui.Loader.PsmlLoader.IsLoading() {
+			opsChan = pcap.OpsChan
+		}
+
 		afterRenderEvents = app.AfterRenderEvents
 
-		prevstate = ui.Loader.State()
+		wasLoadingPdmlLastTime = ui.Loader.PdmlLoader.IsLoading()
+		wasLoadingAnythingLastTime = ui.Loader.LoadingAnything()
 
 		select {
 
@@ -994,19 +981,7 @@ Loop:
 			ui.Fin.Advance()
 			app.Redraw()
 
-		case we := <-tmpPcapWatcherChan:
-			if strings.Contains(we.Name, ifaceTmpFile) {
-				log.Infof("Pcap file %v has appeared - launching UI", we.Name)
-				iwatcher.Close()
-				iwatcher = nil
-				startUIChan <- struct{}{}
-			}
-
-		case err := <-tmpPcapWatcherErrorsChan:
-			fmt.Fprintf(os.Stderr, "Unexpected watcher error for %s: %v", ifaceTmpFile, err)
-			return 1
-
-		case <-startUIChan:
+		case <-ui.StartUIChan:
 			log.Infof("Launching termshark UI")
 
 			// Go to termshark UI view
@@ -1015,6 +990,30 @@ Loop:
 				return 1
 			}
 
+			// Need to do that here because the app won't know how many colors the screen
+			// has (and therefore which variant of the theme to load) until the screen is
+			// activated.
+			mode := theme.Mode(app.GetColorMode()).String() // more concise
+			themeName := termshark.ConfString(fmt.Sprintf("main.theme-%s", mode), "default")
+			loaded := false
+			if themeName != "" {
+				err = theme.Load(themeName, app)
+				if err != nil {
+					log.Warnf("Theme %s could not be loaded: %v", themeName, err)
+				} else {
+					loaded = true
+				}
+			}
+			if !loaded && themeName != "default" {
+				err = theme.Load("default", app)
+				if err != nil {
+					log.Warnf("Theme %s could not be loaded: %v", themeName, err)
+				}
+			}
+
+			// This needs to run after the toml config file is loaded.
+			ui.SetupColors()
+
 			// Start tcell/gowid events for keys, etc
 			appRunner.Start()
 
@@ -1026,10 +1025,7 @@ Loop:
 			ui.Running = true
 			startedSuccessfully = true
 
-			close(startUIChan)
-			startUIChan = nil // make sure it's not triggered again
-
-			close(detectMsgChan) // don't display the message about waiting for the UI
+			ui.StartUIChan = nil // make sure it's not triggered again
 
 			defer func() {
 				// Do this to make sure the program quits quickly if quit is invoked
@@ -1046,9 +1042,13 @@ Loop:
 				ui.Running = false
 			}()
 
+		case fn := <-opsChan:
+			app.Run(fn)
+
 		case <-ui.QuitRequestedChan:
-			quitRequested = true
-			if ui.Loader.State() != 0 {
+			ui.QuitRequested = true
+			// Without this, a quit during a pcap load won't happen until the load is finished
+			if ui.Loader.LoadingAnything() {
 				// We know we're not idle, so stop any load so the quit op happens quickly for the user. Quit
 				// will happen next time round because the quitRequested flag is checked.
 				stopLoaders()
@@ -1097,14 +1097,14 @@ Loop:
 					uiSuspended = false
 				}
 			} else if system.IsSigUSR1(sig) {
-				if cli.FlagIsTrue(opts.Debug) {
+				if debug {
 					termshark.ProfileCPUFor(20)
 				} else {
 					log.Infof("SIGUSR1 ignored by termshark - see the --debug flag")
 				}
 
 			} else if system.IsSigUSR2(sig) {
-				if cli.FlagIsTrue(opts.Debug) {
+				if debug {
 					termshark.ProfileHeap()
 				} else {
 					log.Infof("SIGUSR2 ignored by termshark - see the --debug flag")
@@ -1112,82 +1112,59 @@ Loop:
 
 			} else {
 				log.Infof("Starting termination via signal %v", sig)
-				ui.QuitRequestedChan <- struct{}{}
+				ui.RequestQuit()
 			}
 
-		case fn := <-opsChan:
-			// We run the requested operation - because operations are now enabled, since this channel
-			// is listening - and the result tells us when operations can be re-enabled (i.e. the target
-			// state of the operation just started, for example). This means we can let an operation
-			// "complete", moving through a sequence of states to the final state, befpre accepting
-			// another request.
-			fn()
-
 		case <-ui.CacheRequestsChan:
-			ui.CacheRequests = pcap.ProcessPdmlRequests(ui.CacheRequests, ui.Loader,
-				struct {
-					ui.SetNewPdmlRequests
-					ui.SetStructWidgets
-				}{
-					ui.SetNewPdmlRequests{ui.PcapScheduler},
-					ui.SetStructWidgets{ui.Loader, app},
-				})
-
-		case <-ifaceFinChan:
-			// this state change only happens if the load from the interface is explicitly
-			// stopped by the user (e.g. the stop button). When the current data has come
-			// from loading from an interface, when stopped we still want to be able to filter
-			// on that data. So the load routines should treat it like a regular pcap
-			// (until the interface is started again). That means the psml reader should read
-			// from the file and not the fifo.
-			loaderIfaceFinChan = ui.Loader.IfaceFinishedChan
-			ui.Loader.SetState(ui.Loader.State() & ^pcap.LoadingIface)
-
-		case <-psmlFinChan:
-			if ui.Loader.LoadWasCancelled {
-				// Don't reset cancel state here. If, after stopping an interface load, I
-				// apply a filter, I need to know if the load was cancelled previously because
-				// if it was cancelled, I need to load from the temp pcap; if not cancelled,
-				// (meaning still running), then I just apply a new filter and have the pcap
-				// reader read from the fifo. Only do this if the user isn't quitting the app,
-				// otherwise it looks clumsy.
-				if !quitRequested {
-					app.Run(gowid.RunFunction(func(app gowid.IApp) {
-						ui.OpenError("Loading was cancelled.", app)
-					}))
-				}
-			}
-			// Reset
-			loaderPsmlFinChan = ui.Loader.PsmlFinishedChan
-			ui.Loader.SetState(ui.Loader.State() & ^pcap.LoadingPsml)
-
-		case <-pdmlFinChan:
-			loaderPdmlFinChan = ui.Loader.Stage2FinishedChan
-			ui.Loader.SetState(ui.Loader.State() & ^pcap.LoadingPdml)
+			ui.CacheRequests = pcap.ProcessPdmlRequests(ui.CacheRequests,
+				ui.Loader.ParentLoader, ui.Loader.PdmlLoader, ui.SetStructWidgets{ui.Loader}, app)
 
 		case <-tickChan:
-			if system.HaveFdinfo && (ui.Loader.State() == pcap.LoadingPdml || !ui.Loader.ReadingFromFifo()) {
+			// We already know that we are LoadingPdml|LoadingPsml
+			ui.SetProgressWidget(app)
+			if progCancelTimer != nil {
+				progCancelTimer.Reset(time.Duration(500) * time.Millisecond)
+			} else {
+				progCancelTimer = time.AfterFunc(time.Duration(500)*time.Millisecond, func() {
+					app.Run(gowid.RunFunction(func(app gowid.IApp) {
+						ui.ClearProgressWidget(app)
+					}))
+				})
+			}
+
+			// Rule:
+			// - prefer progress if we can apply it to psml only (not pdml)
+			// - otherwise use a spinner if interface load or fifo load in operation
+			// - otherwise use progress for pdml
+			doprog := false
+			if system.HaveFdinfo {
+				// Prefer progress, if the OS supports it.
+				doprog = true
+				if ui.Loader.ReadingFromFifo() {
+					// But if we are have an interface load (or a pipe load), then we can't
+					// predict when the data will run out, so use a spinner. That's because we
+					// feed the data to tshark -T psml with a tail command which reads from
+					// the tmp file being created by the pipe/interface source.
+					doprog = false
+					if !ui.Loader.InterfaceLoader.IsLoading() && !ui.Loader.PsmlLoader.IsLoading() {
+						// Unless those loads are finished, and the only loading activity is now
+						// PDML/pcap, which is loaded on demand in blocks of 1000. Then we can
+						// use the progress bar.
+						doprog = true
+					}
+				}
+			}
+
+			if doprog {
 				app.Run(gowid.RunFunction(func(app gowid.IApp) {
-					prev = ui.UpdateProgressBarForFile(ui.Loader, prev, app)
+					prevProgPercentage = ui.UpdateProgressBarForFile(ui.Loader, prevProgPercentage, app)
 				}))
 			} else {
 				app.Run(gowid.RunFunction(func(app gowid.IApp) {
-					ui.UpdateProgressBarForInterface(ui.Loader, app)
+					ui.UpdateProgressBarForInterface(ui.Loader.InterfaceLoader, app)
 				}))
 			}
 
-		case <-emptyStructViewChan:
-			app.Run(gowid.RunFunction(func(app gowid.IApp) {
-				ui.SetStructViewMissing(app)
-				ui.StopEmptyStructViewTimer()
-			}))
-
-		case <-emptyHexViewChan:
-			app.Run(gowid.RunFunction(func(app gowid.IApp) {
-				ui.SetHexViewMissing(app)
-				ui.StopEmptyHexViewTimer()
-			}))
-
 		case ev := <-tcellEvents:
 			app.HandleTCellEvent(ev, gowid.IgnoreUnhandledInput)
 			inactivityTimer.Reset(inactiveDuration)
@@ -1202,6 +1179,10 @@ Loop:
 			app.RunThenRenderEvent(ev)
 
 		case <-watcher.ConfigChanged():
+			// Re-read so changes that can take effect immediately do so
+			if err := viper.ReadInConfig(); err != nil {
+				log.Warnf("Unexpected error re-reading toml config: %v", err)
+			}
 			ui.UpdateRecentMenu(app)
 		}
 
