@@ -1,4 +1,4 @@
-// Copyright 2019-2020 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -9,7 +9,9 @@ package pcap
 import (
 	"syscall"
 
+	"github.com/kballard/go-shellquote"
 	"github.com/pkg/errors"
+	log "github.com/sirupsen/logrus"
 )
 
 func (c *Command) PutInNewGroupOnUnix() {
@@ -25,5 +27,7 @@ func (c *Command) Kill() error {
 	if c.Cmd.Process == nil {
 		return errors.WithStack(ProcessNotStarted{Command: c.Cmd})
 	}
-	return syscall.Kill(-c.Cmd.Process.Pid, syscall.SIGINT)
+	log.Infof("Sending SIGKILL to %v: %v", c.Cmd.Process.Pid, shellquote.Join(c.Cmd.Args...))
+	// The PSML tshark process doesn't reliably die with a SIGTERM - not sure why
+	return syscall.Kill(-c.Cmd.Process.Pid, syscall.SIGKILL)
 }
