@@ -1,10 +1,11 @@
-// Copyright 2019-2020 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
 package pcap
 
 import (
+	"fmt"
 	"os"
 	"strings"
 
@@ -26,7 +27,7 @@ type IPacketSource interface {
 func FileSystemSources(srcs []IPacketSource) []IPacketSource {
 	res := make([]IPacketSource, 0)
 	for _, src := range srcs {
-		if src.IsFile() || src.IsFifo() {
+		if src.IsFile() {
 			res = append(res, src)
 		}
 	}
@@ -85,6 +86,10 @@ func (p FileSource) IsPipe() bool {
 	return false
 }
 
+func (p FileSource) String() string {
+	return fmt.Sprintf("File:%s", p.Filename)
+}
+
 //======================================================================
 
 type TemporaryFileSource struct {
@@ -99,6 +104,10 @@ func (p TemporaryFileSource) Remove() error {
 	return os.Remove(p.Filename)
 }
 
+func (p TemporaryFileSource) String() string {
+	return fmt.Sprintf("TempFile:%s", p.Filename)
+}
+
 //======================================================================
 
 type InterfaceSource struct {
@@ -127,6 +136,10 @@ func (p InterfaceSource) IsPipe() bool {
 	return false
 }
 
+func (p InterfaceSource) String() string {
+	return fmt.Sprintf("Interface:%s", p.Iface)
+}
+
 //======================================================================
 
 type FifoSource struct {
@@ -155,6 +168,10 @@ func (p FifoSource) IsPipe() bool {
 	return false
 }
 
+func (p FifoSource) String() string {
+	return fmt.Sprintf("Fifo:%s", p.Filename)
+}
+
 //======================================================================
 
 type PipeSource struct {
@@ -189,6 +206,10 @@ func (p PipeSource) Close() error {
 	return nil
 }
 
+func (p PipeSource) String() string {
+	return fmt.Sprintf("Pipe:%s(%d)", p.Descriptor, p.Fd)
+}
+
 //======================================================================
 // Local Variables:
 // mode: Go
