@@ -1,4 +1,4 @@
-// Copyright 2019-2020 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -98,7 +98,7 @@ type ITrackPayload interface {
 type IIndexerCallbacks interface {
 	IOnStreamChunk
 	ITrackPayload
-	AfterIndexEnd(success bool, closeMe chan<- struct{})
+	AfterIndexEnd(success bool)
 }
 
 func (c *Loader) StartLoad(pcap string, proto string, idx int, app gowid.IApp, cb IIndexerCallbacks) {
@@ -121,55 +121,101 @@ type ISavedData interface {
 func (c *Loader) loadStreamReassemblyAsync(pcapf string, proto string, idx int, app gowid.IApp, cb interface{}) {
 	c.streamCtx, c.streamCancelFn = context.WithCancel(c.mainCtx)
 
+	procChan := make(chan int)
+	pid := 0
+
 	defer func() {
-		c.streamCtx = nil
-		c.streamCancelFn = nil
+		if pid == 0 {
+			close(procChan)
+		}
 	}()
 
 	c.streamCmd = c.cmds.Stream(pcapf, proto, idx)
 
+	termChan := make(chan error)
+
+	termshark.TrackedGo(func() {
+		var err error
+		origCmd := c.streamCmd
+		cancelled := c.streamCtx.Done()
+		procChan := procChan
+		state := pcap.NotStarted
+
+		kill := func() {
+			err := termshark.KillIfPossible(origCmd)
+			if err != nil {
+				log.Infof("Did not kill tshark stream process: %v", err)
+			}
+		}
+
+	loop:
+		for {
+			select {
+			case err = <-termChan:
+				state = pcap.Terminated
+				if !c.SuppressErrors && err != nil {
+					if _, ok := err.(*exec.ExitError); ok {
+						cerr := gowid.WithKVs(termshark.BadCommand, map[string]interface{}{
+							"command": c.streamCmd.String(),
+							"error":   err,
+						})
+						pcap.HandleError(pcap.StreamCode, app, cerr, cb)
+					}
+				}
+
+			case pid := <-procChan:
+				procChan = nil
+				if pid != 0 {
+					state = pcap.Started
+					if cancelled == nil {
+						kill()
+					}
+				}
+
+			case <-cancelled:
+				cancelled = nil
+				if state == pcap.Started {
+					kill()
+				}
+			}
+
+			if state == pcap.Terminated || (cancelled == nil && state == pcap.NotStarted) {
+				break loop
+			}
+		}
+	}, Goroutinewg)
+
 	streamOut, err := c.streamCmd.StdoutReader()
 	if err != nil {
-		pcap.HandleError(err, cb)
+		pcap.HandleError(pcap.StreamCode, app, err, cb)
 		return
 	}
 
-	pcap.HandleBegin(cb)
+	app.Run(gowid.RunFunction(func(app gowid.IApp) {
+		pcap.HandleBegin(pcap.StreamCode, app, cb)
+	}))
 	defer func() {
-		pcap.HandleEnd(cb)
+		app.Run(gowid.RunFunction(func(app gowid.IApp) {
+			pcap.HandleEnd(pcap.StreamCode, app, cb)
+		}))
 	}()
 
 	err = c.streamCmd.Start()
 	if err != nil {
 		err = fmt.Errorf("Error starting stream reassembly %v: %v", c.streamCmd, err)
-		pcap.HandleError(err, cb)
+		pcap.HandleError(pcap.StreamCode, app, err, cb)
 		return
 	}
 
 	log.Infof("Started stream reassembly command %v with pid %d", c.streamCmd, c.streamCmd.Pid())
 
-	defer func() {
-		err = c.streamCmd.Wait() // it definitely started, so we must wait
-		if !c.SuppressErrors && err != nil {
-			if _, ok := err.(*exec.ExitError); ok {
-				cerr := gowid.WithKVs(termshark.BadCommand, map[string]interface{}{
-					"command": c.streamCmd.String(),
-					"error":   err,
-				})
-				pcap.HandleError(cerr, cb)
-			}
-		}
-	}()
-
 	termshark.TrackedGo(func() {
-		// Wait for external cancellation. This is the shutdown procedure.
-		<-c.streamCtx.Done()
-		err := termshark.KillIfPossible(c.streamCmd)
-		if err != nil {
-			log.Infof("Did not kill stream reassembly process: %v", err)
-		}
+		termChan <- c.streamCmd.Wait()
 	}, Goroutinewg)
 
+	pid = c.streamCmd.Pid()
+	procChan <- pid
+
 	var ops []Option
 	ops = append(ops, GlobalStore("app", app))
 	ops = append(ops, GlobalStore("context", c.streamCtx))
@@ -180,64 +226,106 @@ func (c *Loader) loadStreamReassemblyAsync(pcapf string, proto string, idx int,
 			log.Infof("Stream parser reported error: %v", err)
 		}
 	}()
+
+	c.streamCancelFn()
 }
 
 func (c *Loader) startStreamIndexerAsync(pcapf string, proto string, idx int, app gowid.IApp, cb IIndexerCallbacks) {
 	res := false
 
-	c.indexerCtx, c.indexerCancelFn = context.WithCancel(c.mainCtx)
+	procChan := make(chan int)
+	pid := 0
 
 	defer func() {
-		c.indexerCtx = nil
-		c.indexerCancelFn = nil
+		if pid == 0 {
+			close(procChan)
+		}
 	}()
 
+	c.indexerCtx, c.indexerCancelFn = context.WithCancel(c.mainCtx)
+
 	c.indexerCmd = c.cmds.Indexer(pcapf, proto, idx)
 
 	streamOut, err := c.indexerCmd.StdoutReader()
 	if err != nil {
-		pcap.HandleError(err, cb)
+		pcap.HandleError(pcap.StreamCode, app, err, cb)
 		return
 	}
 
+	procWaitChan := make(chan error, 1)
+
+	termshark.TrackedGo(func() {
+		var err error
+		cancelledChan := c.indexerCtx.Done()
+		procChan := procChan
+		state := pcap.NotStarted
+
+		kill := func() {
+			err = termshark.KillIfPossible(c.indexerCmd)
+			if err != nil {
+				log.Infof("Did not kill indexer process: %v", err)
+			}
+		}
+
+	loop:
+		for {
+			select {
+			case err = <-procWaitChan:
+				state = pcap.Terminated
+				if !c.SuppressErrors && err != nil {
+					if _, ok := err.(*exec.ExitError); ok {
+						cerr := gowid.WithKVs(termshark.BadCommand, map[string]interface{}{
+							"command": c.indexerCmd.String(),
+							"error":   err,
+						})
+						pcap.HandleError(pcap.StreamCode, app, cerr, cb)
+					}
+				}
+				streamOut.Close()
+
+			case pid := <-procChan:
+				procChan = nil
+				if pid != 0 {
+					state = pcap.Started
+					if cancelledChan == nil {
+						kill()
+					}
+				}
+
+			case <-cancelledChan:
+				cancelledChan = nil
+				if state == pcap.Started {
+					kill()
+				}
+
+			}
+
+			if state == pcap.Terminated || (cancelledChan == nil && state == pcap.NotStarted) {
+				break loop
+			}
+
+		}
+	}, Goroutinewg)
+
 	defer func() {
-		ch := make(chan struct{})
-		cb.AfterIndexEnd(res, ch)
-		<-ch
+		cb.AfterIndexEnd(res)
 	}()
 
 	err = c.indexerCmd.Start()
 	if err != nil {
 		err = fmt.Errorf("Error starting stream indexer %v: %v", c.indexerCmd, err)
-		pcap.HandleError(err, cb)
+		pcap.HandleError(pcap.StreamCode, app, err, cb)
 		return
 	}
 
 	log.Infof("Started stream indexer command %v with pid %d", c.indexerCmd, c.indexerCmd.Pid())
 
 	defer func() {
-		err = c.indexerCmd.Wait() // it definitely started, so we must wait
-		if !c.SuppressErrors && err != nil {
-			if _, ok := err.(*exec.ExitError); ok {
-				cerr := gowid.WithKVs(termshark.BadCommand, map[string]interface{}{
-					"command": c.indexerCmd.String(),
-					"error":   err,
-				})
-				pcap.HandleError(cerr, cb)
-			}
-		}
+		procWaitChan <- c.indexerCmd.Wait()
 	}()
 
-	termshark.TrackedGo(func() {
-		// Wait for external cancellation. This is the shutdown procedure.
-		<-c.indexerCtx.Done()
-		err := termshark.KillIfPossible(c.indexerCmd)
-		if err != nil {
-			log.Infof("Did not kill indexer process: %v", err)
-		}
-		// Stop main loop
-		streamOut.Close()
-	}, Goroutinewg)
+	pid = c.indexerCmd.Pid()
+	procChan <- pid
 
 	res = decodeStreamXml(streamOut, proto, c.indexerCtx, cb)
 }
