@@ -1,4 +1,4 @@
-// Copyright 2019-2020 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -18,11 +18,11 @@ type IChunk interface {
 }
 
 type IOnStreamChunk interface {
-	OnStreamChunk(chunk IChunk, ch chan struct{})
+	OnStreamChunk(chunk IChunk)
 }
 
 type IOnStreamHeader interface {
-	OnStreamHeader(header FollowHeader, ch chan struct{})
+	OnStreamHeader(header FollowHeader)
 }
 
 //======================================================================
