@@ -1,4 +1,4 @@
-// Copyright 2019-2020 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -6,14 +6,6 @@
 
 package system
 
-import (
-	"fmt"
-)
-
-func MoveStdin() (int, error) {
-	return -1, fmt.Errorf("MoveStdin not implemented on Windows")
-}
-
 func CloseDescriptor(fd int) {
 }
 
