@@ -1,4 +1,4 @@
-// Copyright 2019-2020 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -34,6 +34,7 @@ import (
 	"github.com/gcla/termshark/v2/convs"
 	"github.com/gcla/termshark/v2/pcap"
 	"github.com/gcla/termshark/v2/psmlmodel"
+	"github.com/gcla/termshark/v2/ui/tableutil"
 	"github.com/gcla/termshark/v2/widgets/appkeys"
 	"github.com/gcla/termshark/v2/widgets/copymodetable"
 	"github.com/gcla/termshark/v2/widgets/enableselected"
@@ -120,10 +121,9 @@ type ManageConvsCache struct{}
 var _ pcap.INewSource = ManageConvsCache{}
 
 // Make sure that existing data is discarded if the user loads a new pcap.
-func (t ManageConvsCache) OnNewSource(closeMe chan<- struct{}) {
+func (t ManageConvsCache) OnNewSource(pcap.HandlerCode, gowid.IApp) {
 	convsView = nil // which then deletes all refs to loaded data
 	convsPcapSize = 0
-	close(closeMe)
 }
 
 //======================================================================
@@ -200,11 +200,12 @@ func (p pleaseWait) ClosePleaseWait(app gowid.IApp) {
 	ClosePleaseWait(app)
 }
 
-// Dynamically load conv
+// Dynamically load conv. If the convs window was last opened with a different filter, and the "limit to
+// filter" checkbox is checked, then the data needs to be reloaded.
 func openConvsUi(app gowid.IApp) {
 
 	var convCtx context.Context
-	convCtx, convCancel = context.WithCancel(context.TODO())
+	convCtx, convCancel = context.WithCancel(Loader.Context())
 
 	newSize, reset := termshark.FileSizeDifferentTo(Loader.PcapPdml, convsPcapSize)
 	if reset {
@@ -227,6 +228,8 @@ func openConvsUi(app gowid.IApp) {
 		)
 
 		convsView = holder.New(convsUi)
+	} else if convsUi.FilterValue() != Loader.DisplayFilter() && convsUi.UseFilter() {
+		convsUi.ReloadNeeded()
 	}
 
 	convsUi.ctx = convCtx
@@ -656,7 +659,7 @@ func (w *ConvsUiWidget) doFilterMenuOp(dirOp FilterMask, app gowid.IApp) {
 				w.focusOnFilter = true
 				OpenMessage("Display filter prepared.", appView, app)
 			} else {
-				PcapScheduler.RequestNewFilter(filter, MakePacketViewUpdater(app))
+				RequestNewFilter(filter, app)
 				w.displayFilter = filter
 				OpenMessage("Display filter applied.", appView, app)
 				w.ReloadNeeded()
@@ -704,10 +707,10 @@ func (w *ConvsUiWidget) OnData(data string, app gowid.IApp) {
 				TableSeparator:      divider.NewUnicode(),
 				VerticalSeparator:   nil,
 				CellStyleProvided:   true,
-				CellStyleSelected:   gowid.MakePaletteRef("pkt-list-cell-selected"),
-				CellStyleFocus:      gowid.MakePaletteRef("pkt-list-cell-focus"),
+				CellStyleSelected:   gowid.MakePaletteRef("packet-list-cell-selected"),
+				CellStyleFocus:      gowid.MakePaletteRef("packet-list-cell-focus"),
 				HeaderStyleProvided: true,
-				HeaderStyleFocus:    gowid.MakePaletteRef("pkt-list-cell-focus"),
+				HeaderStyleFocus:    gowid.MakePaletteRef("packet-list-cell-focus"),
 			},
 			Layout: table.LayoutOptions{
 				Widths: wids,
@@ -716,7 +719,7 @@ func (w *ConvsUiWidget) OnData(data string, app gowid.IApp) {
 
 		ptblModel := psmlmodel.New(
 			tblModel,
-			gowid.MakePaletteRef("pkt-list-row-focus"),
+			gowid.MakePaletteRef("packet-list-row-focus"),
 		)
 
 		if currentShortName, ok := convs.OfficialNameToType[cur]; ok {
@@ -732,30 +735,37 @@ func (w *ConvsUiWidget) OnData(data string, app gowid.IApp) {
 
 			boundedTbl := NewRowFocusTableWidget(
 				tbl,
-				"pkt-list-row-selected",
-				"pkt-list-row-focus",
+				"packet-list-row-selected",
+				"packet-list-row-focus",
 			)
 
 			var _ list.IWalker = boundedTbl
 			var _ gowid.IWidget = boundedTbl
 			var _ table.IBoundedModel = tblModel
 
-			w.convs[w.tabIndex[currentShortName]].IWidget = enableselected.New(
-				withscrollbar.New(
-					scrollabletable.New(
-						copymodetable.New(
-							boundedTbl,
-							CsvTableCopier{hdrs, datas},
-							CsvTableCopier{hdrs, datas},
-							"convstable",
-							copyModePalette{},
+			w.convs[w.tabIndex[currentShortName]].IWidget = appkeys.New(
+				enableselected.New(
+					withscrollbar.New(
+						scrollabletable.New(
+							copymodetable.New(
+								boundedTbl,
+								CsvTableCopier{hdrs, datas},
+								CsvTableCopier{hdrs, datas},
+								"convstable",
+								copyModePalette{},
+							),
 						),
+						withscrollbar.Options{
+							HideIfContentFits: true,
+						},
 					),
-					withscrollbar.Options{
-						HideIfContentFits: true,
-					},
 				),
+				tableutil.GotoHandler(&tableutil.GoToAdapter{
+					BoundedWidget: tbl,
+					KeyState:      &keyState,
+				}),
 			)
+
 			w.convs[w.tabIndex[currentShortName]].tbl = tbl
 			w.convs[w.tabIndex[currentShortName]].model = model
 			w.buttonLabels[currentShortName].SetText(fmt.Sprintf(" %s (%d) ", cur, len(datas)), app)
