@@ -1,4 +1,4 @@
-// Copyright 2019-2020 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -54,11 +54,15 @@ type streamKey struct {
 type ManageStreamCache struct{}
 
 var _ pcap.INewSource = ManageStreamCache{}
+var _ pcap.IClear = ManageStreamCache{}
 
 // Make sure that existing stream widgets are discarded if the user loads a new pcap.
-func (t ManageStreamCache) OnNewSource(closeMe chan<- struct{}) {
+func (t ManageStreamCache) OnNewSource(pcap.HandlerCode, gowid.IApp) {
+	clearStreamState()
+}
+
+func (t ManageStreamCache) OnClear(pcap.HandlerCode, gowid.IApp) {
 	clearStreamState()
-	close(closeMe)
 }
 
 //======================================================================
@@ -105,7 +109,7 @@ func startStreamReassembly(app gowid.IApp) {
 	previousFilterValue := FilterWidget.Value()
 
 	FilterWidget.SetValue(filter, app)
-	PcapScheduler.RequestNewFilter(filter, MakePacketViewUpdater(app))
+	RequestNewFilter(filter, app)
 
 	currentStreamKey = &streamKey{proto: proto, idx: streamIndex.Val()}
 
@@ -137,7 +141,7 @@ func startStreamReassembly(app gowid.IApp) {
 
 		// Use the source context. At app shutdown, canceling main will cancel src which will cancel the stream
 		// loader. And changing source should also cancel the stream loader on all occasions.
-		StreamLoader = streams.NewLoader(streams.MakeCommands(), Loader.SourceContext())
+		StreamLoader = streams.NewLoader(streams.MakeCommands(), Loader.Context())
 
 		sh := &streamParseHandler{
 			app:   app,
@@ -177,6 +181,9 @@ type streamParseHandler struct {
 
 var _ streams.IOnStreamChunk = (*streamParseHandler)(nil)
 var _ streams.IOnStreamHeader = (*streamParseHandler)(nil)
+var _ pcap.IBeforeBegin = (*streamParseHandler)(nil)
+var _ pcap.IAfterEnd = (*streamParseHandler)(nil)
+var _ pcap.IOnError = (*streamParseHandler)(nil)
 
 // Run from the app goroutine
 func (t *streamParseHandler) drainChunks() int {
@@ -203,10 +210,12 @@ func (t *streamParseHandler) drainPacketIndices() int {
 	return curLen
 }
 
-func (t *streamParseHandler) BeforeBegin(closeMe chan<- struct{}) {
-	close(closeMe)
-	t.app.Run(gowid.RunFunction(func(app gowid.IApp) {
-		OpenPleaseWait(appView, t.app)
+func (t *streamParseHandler) BeforeBegin(code pcap.HandlerCode, app gowid.IApp) {
+	if code&pcap.StreamCode == 0 {
+		return
+	}
+	app.Run(gowid.RunFunction(func(app gowid.IApp) {
+		OpenPleaseWait(appView, app)
 	}))
 
 	t.tick = time.NewTicker(time.Duration(200) * time.Millisecond)
@@ -218,7 +227,7 @@ func (t *streamParseHandler) BeforeBegin(closeMe chan<- struct{}) {
 	// Start this after widgets have been cleared, to get focus change
 	termshark.TrackedGo(func() {
 		fn := func() {
-			t.app.Run(gowid.RunFunction(func(app gowid.IApp) {
+			app.Run(gowid.RunFunction(func(app gowid.IApp) {
 				t.drainChunks()
 
 				if !t.openedStreams {
@@ -237,7 +246,7 @@ func (t *streamParseHandler) BeforeBegin(closeMe chan<- struct{}) {
 
 	termshark.TrackedGo(func() {
 		fn := func() {
-			t.app.Run(gowid.RunFunction(func(app gowid.IApp) {
+			app.Run(gowid.RunFunction(func(app gowid.IApp) {
 				t.drainPacketIndices()
 			}))
 		}
@@ -253,7 +262,7 @@ func (t *streamParseHandler) BeforeBegin(closeMe chan<- struct{}) {
 		for {
 			select {
 			case <-t.tick.C:
-				t.app.Run(gowid.RunFunction(func(app gowid.IApp) {
+				app.Run(gowid.RunFunction(func(app gowid.IApp) {
 					pleaseWaitSpinner.Update()
 				}))
 			case <-t.stopChunks:
@@ -263,8 +272,7 @@ func (t *streamParseHandler) BeforeBegin(closeMe chan<- struct{}) {
 	}, Goroutinewg)
 }
 
-func (t *streamParseHandler) AfterIndexEnd(success bool, closeMe chan<- struct{}) {
-	close(closeMe)
+func (t *streamParseHandler) AfterIndexEnd(success bool) {
 	t.wid.SetFinished(success)
 	close(t.stopIndices)
 
@@ -275,12 +283,14 @@ func (t *streamParseHandler) AfterIndexEnd(success bool, closeMe chan<- struct{}
 	}
 }
 
-func (t *streamParseHandler) AfterEnd(closeMe chan<- struct{}) {
-	close(closeMe)
-	t.app.Run(gowid.RunFunction(func(app gowid.IApp) {
+func (t *streamParseHandler) AfterEnd(code pcap.HandlerCode, app gowid.IApp) {
+	if code&pcap.StreamCode == 0 {
+		return
+	}
+	app.Run(gowid.RunFunction(func(app gowid.IApp) {
 		if !t.pleaseWaitClosed {
 			t.pleaseWaitClosed = true
-			ClosePleaseWait(t.app)
+			ClosePleaseWait(app)
 		}
 		if !t.openedStreams {
 			openStreamUi(t.wid, app)
@@ -307,28 +317,28 @@ func (t *streamParseHandler) TrackPayloadPacket(packet int) {
 	t.pktIndices <- packet
 }
 
-func (t *streamParseHandler) OnStreamHeader(hdr streams.FollowHeader, ch chan struct{}) {
+func (t *streamParseHandler) OnStreamHeader(hdr streams.FollowHeader) {
 	t.app.Run(gowid.RunFunction(func(app gowid.IApp) {
 		t.wid.AddHeader(hdr)
 	}))
-	close(ch)
 }
 
 // Handle a line/chunk of input - one piece of reassembled data, which comes with
 // a client/server direction.
-func (t *streamParseHandler) OnStreamChunk(chunk streams.IChunk, ch chan struct{}) {
+func (t *streamParseHandler) OnStreamChunk(chunk streams.IChunk) {
 	t.Lock()
 	defer t.Unlock()
 	t.chunks <- chunk
-	close(ch)
 }
 
-func (t *streamParseHandler) OnError(err error, closeMe chan<- struct{}) {
-	close(closeMe)
+func (t *streamParseHandler) OnError(code pcap.HandlerCode, app gowid.IApp, err error) {
+	if code&pcap.StreamCode == 0 {
+		return
+	}
 	log.Error(err)
 	if !Running {
 		fmt.Fprintf(os.Stderr, "%v\n", err)
-		QuitRequestedChan <- struct{}{}
+		RequestQuit()
 	} else {
 
 		var errstr string
@@ -343,7 +353,7 @@ func (t *streamParseHandler) OnError(err error, closeMe chan<- struct{}) {
 			errstr = fmt.Sprintf("%v", err)
 		}
 
-		t.app.Run(gowid.RunFunction(func(app gowid.IApp) {
+		app.Run(gowid.RunFunction(func(app gowid.IApp) {
 			OpenError(errstr, app)
 		}))
 	}
@@ -427,7 +437,7 @@ func (s simpleOnError) OnError(msg string, app gowid.IApp) {
 
 func makeStreamWidget(previousFilter string, filter string, cap string, proto streams.Protocol) *streamwidget.Widget {
 	return streamwidget.New(filter, cap, proto,
-		conversationMenu, conversationMenuHolder,
+		conversationMenu, conversationMenuHolder, &keyState,
 		streamwidget.Options{
 			DefaultDisplay: func() streamwidget.DisplayFormat {
 				view := streamwidget.Hex
@@ -452,7 +462,8 @@ func makeStreamWidget(previousFilter string, filter string, cap string, proto st
 				}
 
 				FilterWidget.SetValue(newFilter, app)
-				PcapScheduler.RequestNewFilter(newFilter, MakePacketViewUpdater(app))
+				//Loader.NewFilter(newFilter, MakePacketViewUpdater(), app)
+				RequestNewFilter(newFilter, app)
 
 			},
 			CopyModeWidget: CopyModeWidget,
