@@ -1,4 +1,4 @@
-// Copyright 2019-2020 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -19,6 +19,7 @@ import (
 
 	"github.com/gcla/gowid"
 	"github.com/gcla/gowid/gwutil"
+	"github.com/gcla/gowid/vim"
 	"github.com/gcla/gowid/widgets/button"
 	"github.com/gcla/gowid/widgets/cellmod"
 	"github.com/gcla/gowid/widgets/columns"
@@ -68,6 +69,7 @@ type Widget struct {
 	quitchan             chan struct{}
 	readytorunchan       chan struct{}
 	temporarilyDisabled  *bool // set to true right after submitting a new filter, so the menu disappears
+	enterPending         bool  // set to true if the user has hit enter; process if the filter goes to valid before another change. For slow validity processing.
 	*gowid.Callbacks
 	gowid.IsSelectable
 }
@@ -86,7 +88,14 @@ type Options struct {
 }
 
 func New(opt Options) *Widget {
+	res := &Widget{}
+
 	ed := edit.New()
+	ed.OnTextSet(gowid.WidgetCallback{"cb", func(app gowid.IApp, w gowid.IWidget) {
+		// every time the filter changes, drop any pending enter - we don't want to
+		// apply a filter to a stale value
+		res.enterPending = false
+	}})
 
 	fixed := gowid.RenderFixed{}
 	filterList := list.New(list.NewSimpleListWalker([]gowid.IWidget{}))
@@ -100,18 +109,27 @@ func New(opt Options) *Widget {
 
 	menuListBox2 := styled.New(
 		framed.NewUnicode(cellmod.Opaque(filterActivator)),
-		gowid.MakePaletteRef("filter-menu-focus"),
+		gowid.MakePaletteRef("filter-menu"),
 	)
 
+	ign := make([]gowid.IKey, 0, len(vim.AllDownKeys)+len(vim.AllUpKeys))
+	for _, k := range vim.AllDownKeys {
+		if !termshark.KeyPressIsPrintable(gowid.Key(k)) {
+			ign = append(ign, gowid.Key(k))
+		}
+	}
+	for _, k := range vim.AllUpKeys {
+		if !termshark.KeyPressIsPrintable(gowid.Key(k)) {
+			ign = append(ign, gowid.Key(k))
+		}
+	}
+
 	drop := menu.New("filter", menuListBox2, gowid.RenderWithUnits{U: opt.MaxCompletions + 2},
 		menu.Options{
 			IgnoreKeysProvided: true,
-			IgnoreKeys: []gowid.IKey{
-				gowid.MakeKeyExt(tcell.KeyUp),
-				gowid.MakeKeyExt(tcell.KeyDown),
-			},
-			CloseKeysProvided: true,
-			CloseKeys:         []gowid.IKey{},
+			IgnoreKeys:         ign,
+			CloseKeysProvided:  true,
+			CloseKeys:          []gowid.IKey{},
 		},
 	)
 
@@ -121,8 +139,7 @@ func New(opt Options) *Widget {
 
 	cb := gowid.NewCallbacks()
 
-	temporarilyDisabled := false
-	onelineEd := appkeys.New(ed, handleEnter(cb, &temporarilyDisabled), appkeys.Options{
+	onelineEd := appkeys.New(ed, handleEnter(cb, res), appkeys.Options{
 		ApplyBefore: true,
 	})
 
@@ -148,7 +165,7 @@ func New(opt Options) *Widget {
 	readytorunchan := make(chan struct{})
 	filterchangedchan := make(chan *filtStruct)
 
-	res := &Widget{
+	*res = Widget{
 		wrapped:              wrapped,
 		opts:                 opt,
 		ed:                   ed,
@@ -166,7 +183,7 @@ func New(opt Options) *Widget {
 		runthisfilterchan:    runthisfilterchan,
 		quitchan:             quitchan,
 		readytorunchan:       readytorunchan,
-		temporarilyDisabled:  &temporarilyDisabled,
+		temporarilyDisabled:  new(bool),
 		Callbacks:            cb,
 	}
 
@@ -175,6 +192,14 @@ func New(opt Options) *Widget {
 			app.Run(gowid.RunFunction(func(app gowid.IApp) {
 				res.validitySite.SetSubWidget(res.valid, app)
 				gowid.RunWidgetCallbacks(res.Callbacks, ValidCB{}, app, res)
+
+				if res.enterPending {
+					var dummy gowid.IWidget
+					gowid.RunWidgetCallbacks(cb, SubmitCB{}, app, dummy)
+					*res.temporarilyDisabled = true
+					res.enterPending = false
+				}
+
 			}))
 		},
 	}
@@ -184,6 +209,7 @@ func New(opt Options) *Widget {
 			app.Run(gowid.RunFunction(func(app gowid.IApp) {
 				res.validitySite.SetSubWidget(res.invalid, app)
 				gowid.RunWidgetCallbacks(res.Callbacks, InvalidCB{}, app, res)
+				res.enterPending = false
 			}))
 		},
 	}
@@ -193,6 +219,7 @@ func New(opt Options) *Widget {
 			app.Run(gowid.RunFunction(func(app gowid.IApp) {
 				res.validitySite.SetSubWidget(res.intermediate, app)
 				gowid.RunWidgetCallbacks(res.Callbacks, IntermediateCB{}, app, res)
+				res.enterPending = false
 			}))
 		},
 	}
@@ -337,18 +364,26 @@ func (f *Validator) Validate(filter string) {
 	}
 }
 
+type iFilterEnter interface {
+	setDisabled()
+	setEnterPending()
+	isValid() bool
+}
+
 // if the filter is valid when enter is pressed, submit the SubmitCB callback. Those
 // registered will be able to respond e.g. start handling the valid filter value.
-func handleEnter(cb *gowid.Callbacks, temporarilyDisabled *bool) appkeys.KeyInputFn {
+func handleEnter(cb *gowid.Callbacks, fe iFilterEnter) appkeys.KeyInputFn {
 	return func(evk *tcell.EventKey, app gowid.IApp) bool {
 		handled := false
 		switch evk.Key() {
 		case tcell.KeyEnter:
-
-			var dummy gowid.IWidget
-			gowid.RunWidgetCallbacks(cb, SubmitCB{}, app, dummy)
-			*temporarilyDisabled = true
-
+			if fe.isValid() {
+				var dummy gowid.IWidget
+				gowid.RunWidgetCallbacks(cb, SubmitCB{}, app, dummy)
+				fe.setDisabled()
+			} else {
+				fe.setEnterPending() // remember in case the filter goes valid shortly
+			}
 			handled = true
 		}
 		return handled
@@ -389,7 +424,7 @@ func newMenuWidgets(ed *edit.Widget, completions []string) []gowid.IWidget {
 				SelectKeys: []gowid.IKey{gowid.MakeKeyExt(tcell.KeyEnter)},
 			},
 		)
-		clickmeStyled := styled.NewInvertedFocus(clickme, gowid.MakePaletteRef("filter-menu-focus"))
+		clickmeStyled := styled.NewInvertedFocus(clickme, gowid.MakePaletteRef("filter-menu"))
 		clickme.OnClick(gowid.MakeWidgetCallback(gowid.ClickCB{}, func(app gowid.IApp, target gowid.IWidget) {
 			txt := ed.Text()
 			end := ed.CursorPos()
@@ -451,6 +486,19 @@ func makeCompletions(comp termshark.IPrefixCompleter, txt string, max int, app g
 	comp.Completions(txt, cb)
 }
 
+func (w *Widget) setDisabled() {
+	*w.temporarilyDisabled = true
+}
+
+func (w *Widget) setEnterPending() {
+	w.enterPending = true
+}
+
+// isCurrentlyValid returns true if the current state of the filter is valid (green)
+func (w *Widget) isValid() bool {
+	return w.validitySite.SubWidget() == w.valid
+}
+
 // Start an asynchronous routine to update the drop-down menu with completion
 // options. Runs on a small delay so it can be cancelled and restarted if the
 // user is typing quickly.
@@ -561,6 +609,7 @@ func (w *Widget) Value() string {
 
 func (w *Widget) SetValue(v string, app gowid.IApp) {
 	w.ed.SetText(v, app)
+	w.ed.SetCursorPos(len(v), app)
 }
 
 func (w *Widget) Menus() []gowid.IMenuCompatible {
@@ -588,7 +637,7 @@ func (w *Widget) Render(size gowid.IRenderSize, focus gowid.Selector, app gowid.
 // it go orange briefly, which is unpleasant.
 func (w *Widget) UserInput(ev interface{}, size gowid.IRenderSize, focus gowid.Selector, app gowid.IApp) bool {
 	if evk, ok := ev.(*tcell.EventKey); ok {
-		if evk.Key() == tcell.KeyTAB || evk.Key() == tcell.KeyDown {
+		if evk.Key() == tcell.KeyTAB || (vim.KeyIn(evk, vim.AllDownKeys) && !termshark.KeyPressIsPrintable(evk)) {
 			return false
 		}
 	}
@@ -611,7 +660,7 @@ type activatorWidget struct {
 
 func (w *activatorWidget) UserInput(ev interface{}, size gowid.IRenderSize, focus gowid.Selector, app gowid.IApp) bool {
 	if ev, ok := ev.(*tcell.EventKey); ok && !w.active {
-		if ev.Key() == tcell.KeyDown {
+		if vim.KeyIn(ev, vim.AllDownKeys) && !termshark.KeyPressIsPrintable(ev) {
 			w.active = true
 			return true
 		} else {
@@ -621,7 +670,7 @@ func (w *activatorWidget) UserInput(ev interface{}, size gowid.IRenderSize, focu
 	res := w.IWidget.UserInput(ev, size, focus, app)
 	if !res {
 		if ev, ok := ev.(*tcell.EventKey); ok && w.active {
-			if ev.Key() == tcell.KeyUp {
+			if vim.KeyIn(ev, vim.AllUpKeys) && !termshark.KeyPressIsPrintable(ev) {
 				w.active = false
 				return true
 			} else {
