@@ -1,4 +1,4 @@
-// Copyright 2019-2020 Graham Clark. All rights reserved.  Use of this source
+// Copyright 2019-2021 Graham Clark. All rights reserved.  Use of this source
 // code is governed by the MIT license that can be found in the LICENSE
 // file.
 
@@ -27,6 +27,11 @@ func SwallowMovementKeys(ev *tcell.EventKey, app gowid.IApp) bool {
 	switch ev.Key() {
 	case tcell.KeyDown, tcell.KeyCtrlN, tcell.KeyUp, tcell.KeyCtrlP, tcell.KeyRight, tcell.KeyCtrlF, tcell.KeyLeft, tcell.KeyCtrlB:
 		res = true
+	case tcell.KeyRune:
+		switch ev.Rune() {
+		case 'h', 'j', 'k', 'l':
+			res = true
+		}
 	}
 	return res
 }
